//------------------------------------------------------------------------------------------------------------------------
// Class       : BP::CL_BP_Map
// Author      : BehemothProgrammer
// Description : Map manager actor that should be spawned in via script only and never placed in the map itself with the editor.
// OnTrigger   : Nothing.
//
// Parameters  : 
//------------------------------------------------------------------------------------------------------------------------
namespace BP
{
	int mapActorCount;

	class CL_BP_Map : CL_BP_ScriptObject
	{
		kActor@ musicSource; //The musics actor audio source
		int musicLastFlags; //the previous frames music flags
		array<kStr> musicTracks; //list of tracks sound shaders
		array<float> musicTracksDuration; //list of tracks durations
		int musicMapID; //the mapID the music can play on
		int musicPlayOrder; //current track play order
		int musicTrackIndex; //current track index playing
		float musicTrackTime; //amount of time has been playing the track
		kStr musicTrackPath; //current track sound shader path
		array<int> musicAvailTrackIndexs; //list of track indexes available for unique random play order 
		int musicTracksPlayed; //number of tracks played
		float hudHelpTime;
		kStr hudHelpMsg;
		//------------------------------------------------------------------------------------------------------------------------
		CL_BP_Map(kActor @actor)
		{
			super(actor);
			if (mapActorCount > 0)
			{
				self.Remove();
				return;
			}
			
			mapActorCount++;
			MapStartSetup();
			int musicFlags = BP::GameVar::GetInt(GV_KEY_MUSICFLAGS);
			if (musicFlags & BP::Music::MF_STOPONNEXTMAP != 0)
			{
				BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, BP::Music::MF_STOPONNEXTMAP, false);
				BP::Music::Stop();
			}
			BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, BP::Music::MF_LISTUPDATE);
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnTick() override
		{
			BP::GameVar::SetFloat(GV_KEY_GAMETIME, BP::Time::GameTime() + GAME_DELTA_TIME);
			MusicTick();
			BP::Input::Tick();
			HUDTick();
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnEndLevel()
		{
			mapActorCount = 0;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void HUDTick()
		{
			//Update Help Message
			int gvHelpTime = BP::GameVar::GetInt(GV_KEY_HUDHELPTIME);
			if (gvHelpTime > 0)
			{
				BP::GameVar::SetInt(GV_KEY_HUDHELPTIME, 0);
				hudHelpMsg = BP::GameVar::GetString(GV_KEY_HUDHELP);
				hudHelpTime = BP::Time::FromNow(gvHelpTime / 60.0f);
				
				//check for \n characters and convert them to char 10 (new line) which kStr does automatically by converting "\n" to char 10
				array<kStr> msgLines = BP::String::Split(hudHelpMsg, "\\n");
				int lineCount = int(msgLines.length());
				if (lineCount > 1)
				{
					hudHelpMsg = "";
					for (int i = 0; i < lineCount - 1; i++)
						hudHelpMsg += msgLines[i] + "\n";
					hudHelpMsg += msgLines[lineCount - 1];
				}
			}
			
			if (hudHelpTime > 0.0f)
			{
				Game.PrintHelp(hudHelpMsg);
				if (BP::Time::Reached(hudHelpTime))
				{
					hudHelpTime = 0.0f;
					hudHelpMsg = "";
					Game.PrintHelp("");
				}
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
		void MusicTick()
		{
			int musicFlags = BP::GameVar::GetInt(GV_KEY_MUSICFLAGS);
			//On flags changed
			if (musicLastFlags != musicFlags)
			{
				musicLastFlags = musicFlags;
				if ((musicFlags & BP::Music::MF_LISTUPDATE) != 0)
				{
					BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, BP::Music::MF_LISTUPDATE, false);
					
					bool gotEventData = BP::Music::GetEventData(musicTracks, musicTracksDuration, musicPlayOrder, musicMapID);
					if (gotEventData)
					{
						musicTracksPlayed = 0;
						musicAvailTrackIndexs.resize(0);
						switch (musicPlayOrder)
						{
							case BP::Music::MUSIC_PO_RANDOM:
							{
								MusicSetTrack(BP::Array::GetUniqueElement(musicTracks.length(), musicAvailTrackIndexs));
								break;
							}
							default:
							{
								MusicSetTrack(0);
								break;
							}
						}
					}
				}
			}

			float musicVolume = BP::CVar::GetFloat(CVAR_MUSICVOLUME);
			bool canPlay = !((musicFlags & BP::Music::MF_ISPLAYING) == 0 || (musicFlags & BP::Music::MF_DISABLE) != 0 || BP::String::IsEmpty(musicTrackPath) || 
							(musicMapID != -2 && musicMapID != Game.GetCurrentMapID()) || (BP::Cinema::IsActive() && BP::Actor::IsDead(playerActor)) || musicVolume < 0.1f);
			if (canPlay)
			{
				if (!BP::Actor::Exists(musicSource))
				{
					@musicSource = BP::Actor::Spawn(ACTOR_NAME_MUSIC);
				}
				else
				{
					musicSource.Origin() = (BP::Cinema::IsActive() ? Camera.origin : playerActor.Origin());
					musicSource.PlaySound(musicTrackPath);
				}
				
				float lastTrackTime = musicTrackTime;
				musicTrackTime += GAME_DELTA_TIME;
				if (musicTracksDuration[musicTrackIndex] > 0.0f && musicTrackTime >= musicTracksDuration[musicTrackIndex])
				{
					musicTracksPlayed++;
					switch (musicPlayOrder)
					{
						case BP::Music::MUSIC_PO_RANDOM:
						case BP::Music::MUSIC_PO_RANDOMAFTERFIRST:
						{
							MusicSetTrack(BP::Array::GetUniqueElement(musicTracks.length(), musicAvailTrackIndexs, musicTrackIndex));
							break;
						}
						default:
						{
							MusicSetTrack(BP::Array::NextIndex(musicTracks.length(), musicTrackIndex));
							break;
						}
					}
				}
			}
			else
			{
				musicTrackTime = 0.0f;
				if (BP::Actor::Exists(musicSource))
				{
					musicSource.StopLoopingSounds();
				}
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
		void MusicSetTrack(const int index)
		{
			musicTrackIndex = index;
			musicTrackPath = musicTracks[musicTrackIndex];
			musicTrackTime = 0.0f;
			if (BP::Actor::Exists(musicSource))
			{
				musicSource.StopLoopingSounds();
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
	};
}
